/* Name: main.c
 * Project: DeskLamp
 * Author: Stefan Krupop
 * Creation Date: 2008
 * Tabsize: 4
 */

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

#include "usbconfig.h"
#include "usbdrv/usbdrv.h"
#include "libs-device/osccal.h"

#define PWM_WIDTH			0xa0
#define PWM_PORT			PORTB

#define PIN_PWM_Lamp		4


static volatile uchar lampIntensity = 255;
static uchar g_PWM_SawTooth = 0;


#define UTIL_INTERRUPT(signame)                         \
    void signame (void) __attribute__ ((interrupt));    \
    void signame (void)


static void initHardware(void) {
    DDRB |= (1 << 1) | (1 << 4);					// output for FET

	//timer 0
	TCCR0B = 0b00000010;							// set clk/8
	TIMSK |= (1 << TOIE0);
	TIFR = 0;										// clear interrupt flags
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB Interface ----------------------------- */
/* ------------------------------------------------------------------------- */

#define LOWBYTE(var)    (((uchar *)&(var))[0])
#define HIGHBYTE(var)   (((uchar *)&(var))[1])

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x05, 0x08,                    // USAGE_PAGE (LEDs)
    0x09, 0x23,                    // USAGE (Night Mode)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x3d,                    //   USAGE (Indicator On)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x85, 0x01,                    //   REPORT_ID (1)
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x91, 0x02,                    //   OUTPUT (Data,Var,Abs)
    0xc0                           // END_COLLECTION
};

uchar usbFunctionRead(uchar *data, uchar len) {
    *data++ = lampIntensity;
    return 1;
}

uchar usbFunctionWrite(uchar *data, uchar len) {
	lampIntensity = data[1];
    return 1;
}

uchar   usbFunctionSetup(uchar data[8]) {
	static uchar    replyBuf[3];
	usbRequest_t    *rq = (void *)data;

    usbMsgPtr = replyBuf;
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* class request type */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
			return 0;
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            if(rq->wValue.bytes[0] == 1){   /* ReportID "SET BRIGHTNESS" */
                return 0xff;                /* use usbFunctionWrite() */
            }
			return 0;
        }
    }else{
        if(data[1] == 0){       /* ECHO, for testing */
            replyBuf[0] = data[2];
            replyBuf[1] = data[3];
            return 2;
        }
    }
    return 0;
}

void usbEventResetReady(void) {
	cli();  // usbMeasureFrameLength() counts CPU cycles, so disable interrupts.
    calibrateOscillator();
    sei();
    eeprom_write_byte(0, OSCCAL);   // store the calibrated value in EEPROM
}

/* ------------------------------------------------------------------------- */

int main(void) {
	uchar   i;
	uchar   calibrationValue;

    calibrationValue = eeprom_read_byte(0); // calibration value from last time
    if(calibrationValue != 0xff){
        OSCCAL = calibrationValue;
    }

	usbDeviceDisconnect();
    for(i=0;i<20;i++){  					// 300 ms disconnect
		_delay_ms(15);
    }
    usbDeviceConnect();
    initHardware();
    usbInit();
    sei();
    for(;;){    							// main event loop
		usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */

UTIL_INTERRUPT (SIG_OVERFLOW0) {
	if (lampIntensity == 0) {
		PWM_PORT &= ~(1 << PIN_PWM_Lamp);
	} else if (lampIntensity >= g_PWM_SawTooth) {
		PWM_PORT |= 1 << PIN_PWM_Lamp;
	} else {
		PWM_PORT &= ~(1 << PIN_PWM_Lamp);
	}
	g_PWM_SawTooth++;
	TCNT0 = PWM_WIDTH;
}
